#!/bin/sh
#
# Copyright(c) 1998, Space Science and Engineering Center, UW-Madison
# Refer to "McIDAS Software Acquisition and Distribution Policies"
# in the file  mcidas/data/license.txt
#
#	$Id: mcidas.sh,v 1.70 2020/03/06 21:03:15 davep Exp $
#
#-------
# This script starts a mcidas session with the specified parameters.
#
# If the environment variable MCIDASRC is set to the name of a file,
# the mcidas script reads it instead of $HOME/.mcidasrc for options.
#-------

#-------
# Set some defaults.
#-------

error_stat=FALSE

mcidasrc=$HOME/.mcidasrc	# name of file with saved options

: ${MCIDASRC:=$mcidasrc}	# allow the user to override the name
				# of the file with the saved options

cmdhistlen_def=100		# DEFcmdhistlen from mct.c

winhistlen_def=1000		# DEFwinhistlen from mct.c

fontListHead_def='5x8,6x9,6x10,6x12,6x13bold,7x13bold,8x13bold,7x14bold'
				# from mct_winx.c

fontname_def='9x15bold'		# from mct_winx.c

fontListTail_def='8x16,10x20,12x24'
				# from mct_winx.c

tcolbg0_def=black		# from mct_winx.c, m0tcolor.c, mcterm.w
tcolbg16_def=blue4		# from mct_winx.c, m0tcolor.c, mcterm.w
tcolbg32_def=green4		# from mct_winx.c, m0tcolor.c, mcterm.w
tcolbg48_def=cyan4		# from mct_winx.c, m0tcolor.c, mcterm.w
tcolbg64_def=red4		# from mct_winx.c, m0tcolor.c, mcterm.w
tcolbg80_def=magenta4		# from mct_winx.c, m0tcolor.c, mcterm.w
tcolbg96_def=yellow4		# from mct_winx.c, m0tcolor.c, mcterm.w
tcolbg112_def=grey80		# from mct_winx.c, m0tcolor.c, mcterm.w

tcolfg0_def=black		# from mct_winx.c, m0tcolor.c, mcterm.w
tcolfg1_def=blue4		# from mct_winx.c, m0tcolor.c, mcterm.w
tcolfg2_def=green4		# from mct_winx.c, m0tcolor.c, mcterm.w
tcolfg3_def=cyan4		# from mct_winx.c, m0tcolor.c, mcterm.w
tcolfg4_def=red4		# from mct_winx.c, m0tcolor.c, mcterm.w
tcolfg5_def=magenta4		# from mct_winx.c, m0tcolor.c, mcterm.w
tcolfg6_def=yellow4		# from mct_winx.c, m0tcolor.c, mcterm.w
tcolfg7_def=grey80		# from mct_winx.c, m0tcolor.c, mcterm.w

tcolfg8_def=grey50		# from mct_winx.c, m0tcolor.c, mcterm.w
tcolfg9_def=blue		# from mct_winx.c, m0tcolor.c, mcterm.w
tcolfg10_def=green		# from mct_winx.c, m0tcolor.c, mcterm.w
tcolfg11_def=cyan		# from mct_winx.c, m0tcolor.c, mcterm.w
tcolfg12_def=red		# from mct_winx.c, m0tcolor.c, mcterm.w
tcolfg13_def=magenta		# from mct_winx.c, m0tcolor.c, mcterm.w
tcolfg14_def=yellow		# from mct_winx.c, m0tcolor.c, mcterm.w
tcolfg15_def=white		# from mct_winx.c, m0tcolor.c, mcterm.w

tgeometry_def='85x26+300+400'	# from mct_winx.c

igeometry_def='+0+0'		# from mci.c

case $DISPLAY in
'')	DISPLAY_def="(not set)"
	;;
*)	DISPLAY_def="$DISPLAY"
	;;
esac

wait_arg=			# whether to wait for mcenv

verbose_arg=			# if nonempty, echo the command

tty_arg=			# if nonempty, start a tty mcidas
				# session instead of an X mcidas
				# session

# check user

case `id` in
'uid=0('*)
	echo "ERROR: cannot execute as user 'root'"
	exit 1
	;;
uid=?'(mcidas)'*	|\
uid=??'(mcidas)'*	|\
uid=???'(mcidas)'*	|\
uid=????'(mcidas)'*	|\
uid=?????'(mcidas)'*	|\
uid=??????'(mcidas)'*	|\
uid=???????'(mcidas)'*	)
	echo "ERROR: cannot execute as user 'mcidas'"
	exit 1
	;;
esac

#-------
# If there is no mcidasrc file, we create a mcidasrc file with default
# contents.
#-------

if	[ ! -f "$mcidasrc" ]
then
	echo "Creating options file $mcidasrc"
	cat <<!EOF! > $mcidasrc
#-------
# \$HOME/.mcidasrc - options file for the McIDAS-X mcidas shell script
#-------

#-------
# Set the default frame space.
#
# -f framespec
#
#	This option allocates frames according to the framespec.
#	The framespec may have the following forms:
#
#		COUNT@LINESxELEMS	COUNT frames of size LINESxELEMS
#		LINESxELEMS		1 frame of size LINESxELEMS
#		COUNT			COUNT frames of size 480x640
#
# 	This option may be repeated as many times as desired,
# 	in order to make any desired set of frames.
#
#	The order of the -f options determines the frame numbers.
#	The frames are numbered starting with 1.
#
#	For example,
#
# 		-f 4
# 		-f 512x512
# 		-f 512x512
# 		-f 2@1024x1280
#
#	causes your session to have 8 frames as follows.
#
#		Frames 1-4 are  480 lines by  640 elements.
#		Frames 5-6 are  512 lines by  512 elements.
#		Frames 7-8 are 1024 lines by 1280 elements.
#
#	The default is to have four frames of size 480x640.
#-------

-f 10

#-------
# Set the MAKFRM pool size.
#
# -e COUNT
#
#	This option allocates a pool of COUNT bytes for MAKFRM to use
#	to define new frames during a session.
#
# 	If COUNT ends in 'k' it counts KB (e.g. '256k').
# 	If COUNT ends in 'm' it counts MB (e.g. '10m').
#
# 	If this option is given multiple times, only the last instance
# 	is used.
#
#	For example,
#
#		-e 5m
#
#	allocates a 5 MB memory pool for MAKFRM to use.
#
#	Historically, the reason to have MAKFRM was because it was the
#	only way to have frames of different sizes in your session.
#	Because you can now create frames of arbitrary sizes at startup
#	using -f options, the default size of the MAKFRM pool is 0.
#
#	If you want to have a 10 MB pool, simply uncomment the -e line
#	below.
#-------

# -e 10m

#-------
# Run commands on startup.
#
# -c 'command'
#
#	This option causes mctext to run the given command on startup.
#
# 	This option may be repeated as many times as desired,
# 	in order to run any desired set of commands.
#
# 	Each -c argument is executed asynchronously from all the other
#	-c arguments, just as if you had typed it into mctext.
# 	This means that if you need to run certain commands in sequence,
# 	you need to specify them all in one -c option.
#
#	For example,
#
#		-c 'ECHO {HELLO}; ECHO "THERE'
#
#	runs the two ECHO commands in sequence, just as if you had
#	typed them in yourself.
#
#	If the -c argument begins with a '!', the command is executed
#	by /bin/sh.  If the '!' is followed by an '@', the command is
#	not echoed.
#-------

# Run mccheck, in order to check that you have no per-session files
# in non-per-session directories.  For instance, FRAMED.001,
# TERMCHAR.001 and SYSIMAGE.SAV are per-session files, and it is not
# wise to have them occur in directories in your MCPATH.
#
# However, since these are only problems when you are actually running
# two sessions at once, mccheck only prints its warnings when you are
# running at least two sessions at once.
#
# The ! means to run it with the shell, the @ means not to echo it,
# and the exec saves a process.
#
# Remove this option only if you know what you are doing.

-c '!@exec mccheck'

# By default, mctext inverts the case of letters you type.
# If you wish to turn off this behavior, uncomment the following line.

# -c 'CASEMODE NONE'

# By default, there is a scheduler.
#
# Note, however, that if you have two sessions running at once,
# they will each have a scheduler.  If the two sessions use the
# same MCPATH, the schedulers will use the same schedule file.
# This may not be what you want.
#
# If you do not wish to start the scheduler with the default schedule
# file, comment out the -c line below.

-c 'SKED SKEDFILE'

# By default, start the GUI
#
# If you do not wish to start the GUI, comment out the -c line below.

-c GUI

#-------
# Set the text fonts.
#	mctext has a list of fonts that it can use.
#	You can change which font in the list it uses by
#	using Alt-< and Alt->.
#	The list is built up from the fonts specified in the
#	-fontListHead, -font, and -fontListTail options.
#	The default font is the one given by the -font option.
#
#	Note that in order for mctext to work properly,
#	the fonts must be character-cell fonts.
#	For best results, you should restrict yourself
#	to the fonts listed under the description of -font.
#-------

#-------
# Set the head of the text font list.
#
# -fontListHead 'fontlist'
#
#	This option sets the head of the font list that mctext
#	can use in the command window.
#
#	The fonts in the list are separated by commas.
#
#	Note that in order for mctext to work properly,
#	the fonts in the list must be a character-cell fonts.
#-------

# -fontListHead '$fontListHead_def'

#-------
# Set the default text font.
#
# -font 'fontname'
#
#	This option sets the font that mctext uses in the command
#	window.
#
#	The default fontname is '$fontname_def'.
#	Another nice font that is somewhat larger is '10x20'.
#-------

# These character cell fonts are universally available.
# They are listed in order of increasing height, then increasing
# width and weight for the same height.

# -font '5x8'
# -font '6x9'
# -font '6x10'
# -font '6x12'
# -font '6x13'
# -font '6x13bold'
# -font '7x13'
# -font '7x13bold'
# -font '8x13'
# -font '8x13bold'
# -font '7x14'
# -font '7x14bold'
# -font '9x15'
# -font '9x15bold'
# -font '8x16'
# -font '10x20'
# -font '12x24'

# These character cell fonts are also universally available.
# They are listed in order of increasing height, then increasing
# width and weight for the same height.

# -font '-schumacher-clean-medium-r-normal--6-*-*-*-c-40-iso8859-1'
# -font '-schumacher-clean-medium-r-normal--6-*-*-*-c-50-iso8859-1'
# -font '-schumacher-clean-medium-r-normal--6-*-*-*-c-60-iso8859-1'
# -font '-schumacher-clean-medium-r-normal--8-*-*-*-c-50-iso8859-1'
# -font '-schumacher-clean-medium-r-normal--8-*-*-*-c-60-iso8859-1'
# -font '-schumacher-clean-medium-r-normal--8-*-*-*-c-70-iso8859-1'
# -font '-schumacher-clean-medium-r-normal--8-*-*-*-c-80-iso8859-1'
# -font '-schumacher-clean-bold-r-normal--8-*-*-*-c-80-iso8859-1'
# -font '-schumacher-clean-medium-r-normal--10-*-*-*-c-50-iso8859-1'
# -font '-schumacher-clean-medium-r-normal--10-*-*-*-c-60-iso8859-1'
# -font '-schumacher-clean-bold-r-normal--10-*-*-*-c-60-iso8859-1'
# -font '-schumacher-clean-medium-r-normal--10-*-*-*-c-70-iso8859-1'
# -font '-schumacher-clean-medium-r-normal--10-*-*-*-c-80-iso8859-1'
# -font '-schumacher-clean-bold-r-normal--10-*-*-*-c-80-iso8859-1'
# -font '-schumacher-clean-medium-r-normal--12-*-*-*-c-60-iso8859-1'
# -font '-schumacher-clean-bold-r-normal--12-*-*-*-c-60-iso8859-1'
# -font '-schumacher-clean-medium-r-normal--12-*-*-*-c-70-iso8859-1'
# -font '-schumacher-clean-medium-r-normal--12-*-*-*-c-80-iso8859-1'
# -font '-schumacher-clean-bold-r-normal--12-*-*-*-c-80-iso8859-1'
# -font '-schumacher-clean-medium-r-normal--13-*-*-*-c-60-iso8859-1'
# -font '-schumacher-clean-medium-r-normal--13-*-*-*-c-80-iso8859-1'
# -font '-schumacher-clean-bold-r-normal--13-*-*-*-c-80-iso8859-1'
# -font '-schumacher-clean-medium-r-normal--14-*-*-*-c-70-iso8859-1'
# -font '-schumacher-clean-medium-r-normal--14-*-*-*-c-80-iso8859-1'
# -font '-schumacher-clean-bold-r-normal--14-*-*-*-c-80-iso8859-1'
# -font '-schumacher-clean-medium-r-normal--15-*-*-*-c-90-iso8859-1'
# -font '-schumacher-clean-bold-r-normal--15-*-*-*-c-90-iso8859-1'
# -font '-schumacher-clean-medium-r-normal--16-*-*-*-c-80-iso8859-1'
# -font '-schumacher-clean-bold-r-normal--16-*-*-*-c-80-iso8859-1'

#-------
# Set the tail of the text font list.
#
# -fontListTail 'fontlist'
#
#	This option sets the tail of the font list that mctext
#	can use in the command window.
#
#	The fonts in the list are separated by commas.
#
#	Note that in order for mctext to work properly,
#	the fonts in the list must be a character-cell fonts.
#-------

# -fontListTail '$fontListTail_def'

#-------
# The following is an example of a font configuration that uses only
# schumacher fonts, and has the default font as the last font in the
# list.
#-------

# -fontListHead '
# 	-schumacher-clean-medium-r-normal--6-*-*-*-c-40-iso8859-1,
# 	-schumacher-clean-medium-r-normal--6-*-*-*-c-50-iso8859-1,
# 	-schumacher-clean-medium-r-normal--6-*-*-*-c-60-iso8859-1,
# 	-schumacher-clean-medium-r-normal--8-*-*-*-c-50-iso8859-1,
# 	-schumacher-clean-medium-r-normal--8-*-*-*-c-60-iso8859-1,
# 	-schumacher-clean-medium-r-normal--8-*-*-*-c-70-iso8859-1,
# 	-schumacher-clean-medium-r-normal--8-*-*-*-c-80-iso8859-1,
# 	-schumacher-clean-medium-r-normal--10-*-*-*-c-50-iso8859-1,
# 	-schumacher-clean-medium-r-normal--10-*-*-*-c-60-iso8859-1,
# 	-schumacher-clean-medium-r-normal--10-*-*-*-c-70-iso8859-1,
# 	-schumacher-clean-medium-r-normal--10-*-*-*-c-80-iso8859-1,
# 	-schumacher-clean-medium-r-normal--12-*-*-*-c-60-iso8859-1,
# 	-schumacher-clean-medium-r-normal--12-*-*-*-c-70-iso8859-1,
# 	-schumacher-clean-medium-r-normal--12-*-*-*-c-80-iso8859-1,
# 	-schumacher-clean-medium-r-normal--13-*-*-*-c-60-iso8859-1,
# 	-schumacher-clean-medium-r-normal--13-*-*-*-c-80-iso8859-1,
# 	-schumacher-clean-medium-r-normal--14-*-*-*-c-70-iso8859-1,
# 	-schumacher-clean-medium-r-normal--14-*-*-*-c-80-iso8859-1,
# 	-schumacher-clean-medium-r-normal--15-*-*-*-c-90-iso8859-1'
# 
# -font '-schumacher-clean-medium-r-normal--16-*-*-*-c-80-iso8859-1'
# 
# -fontListTail ''

#-------
# Set the number of commands to remember.
#
# -ih number
#
#	This option sets the number of commands that mctext remembers.
#
#	You can recall commands using the command editor.
#	For instance, '^' and '&' in the default command editor.
#
#	The default number of commands that mctext remembers is $cmdhistlen_def.
#-------

# -ih $cmdhistlen_def

#-------
# Set the number of output lines to remember.
#
# -oh number
#
#	This option sets the maximum number of output lines that mctext
#	remembers for each logical text window.
#
#	By default, mctext remembers up to $winhistlen_def lines of
#	command output in each logical text window.
#-------

# -oh $winhistlen_def

#-------
# Set the appearance of the colors in the text window.
#
# -tcolbg0	color	# for background black
# -tcolbg16	color	# for background blue
# -tcolbg32	color	# for background green
# -tcolbg48	color	# for background cyan
# -tcolbg64	color	# for background red
# -tcolbg80	color	# for background magenta
# -tcolbg96	color	# for background yellow
# -tcolbg112	color	# for background white
# 
# -tcolfg0	color	# for foreground black
# -tcolfg1	color 	# for foreground blue
# -tcolfg2	color 	# for foreground green
# -tcolfg3	color 	# for foreground cyan
# -tcolfg4	color 	# for foreground red
# -tcolfg5	color 	# for foreground magenta
# -tcolfg6	color 	# for foreground yellow
# -tcolfg7	color 	# for foreground white
#
# -tcolfg8	color 	# for foreground bright black
# -tcolfg9	color 	# for foreground bright blue
# -tcolfg10	color 	# for foreground bright green
# -tcolfg11	color 	# for foreground bright cyan
# -tcolfg12	color 	# for foreground bright red
# -tcolfg13	color 	# for foreground bright magenta
# -tcolfg14	color 	# for foreground bright yellow
# -tcolfg15	color 	# for foreground bright white
#
#	Every character in the mctext window has a color attribute that
#	specifies one of 16 logical foreground colors (numbered 0..15)
#	and one of 8 logical background colors (numbered 16*0..16*7).
#	The color attribute is the sum of the logical foreground and
#	background color numbers.  You can cause the text output of a
#	McIDAS command to use one of these attributes by specifying the
#	TCOL= keyword on the command line.
#
#	For instance, the attribute value 97 specifies background color
#	96 (which defaults to yellow) and foreground color 1 (which
#	defaults to blue).  The attribute value 7 specifies background
#	color 0 (which defaults to black) and foreground color 7 (which
#	defaults to white).
#
#	These options allow you to change the correspondence between the
#	logical colors and the actual colors.  For example, if you want
#	the logical background color black to display as blue, use this
#	option:
#
#		-tcolbg0 blue
#
#	The color specifications follow the X conventions,
#	as described in the X(1) man page.
#
#	Here are some uses the system makes of certain attributes:
#
#		7		local text
#		10		echoed commands
#		11		command editor
#		13		communications status (e.g. from IMPORT)
#		14		EDEST output
#		15		mainframe text
#		96		line continuation marks
#-------

# -tcolbg0	'$tcolbg0_def'
# -tcolbg16	'$tcolbg16_def'
# -tcolbg32	'$tcolbg32_def'
# -tcolbg48	'$tcolbg48_def'
# -tcolbg64	'$tcolbg64_def'
# -tcolbg80	'$tcolbg80_def'
# -tcolbg96	'$tcolbg96_def'
# -tcolbg112	'$tcolbg112_def'

# -tcolfg0	'$tcolfg0_def'
# -tcolfg1	'$tcolfg1_def'
# -tcolfg2	'$tcolfg2_def'
# -tcolfg3	'$tcolfg3_def'
# -tcolfg4	'$tcolfg4_def'
# -tcolfg5	'$tcolfg5_def'
# -tcolfg6	'$tcolfg6_def'
# -tcolfg7	'$tcolfg7_def'

# -tcolfg8	'$tcolfg8_def'
# -tcolfg9	'$tcolfg9_def'
# -tcolfg10	'$tcolfg10_def'
# -tcolfg11	'$tcolfg11_def'
# -tcolfg12	'$tcolfg12_def'
# -tcolfg13	'$tcolfg13_def'
# -tcolfg14	'$tcolfg14_def'
# -tcolfg15	'$tcolfg15_def'

#-------
# Set the X display to use.
#
# -display name
#
#	This option sets the X display for the McIDAS-X session
#	to use.  Typically you will not need to set this here,
#	because you will have DISPLAY in your environment.
#-------

# -display unix:0.0

#-------
# Set the visual mode for McIDAS.
#
# -displayVisualMode
#
#       This option forces McIDAS to run in the visual mode set for
#       the workstation display, e.g., TrueColor, StaticColor, etc.
#       The default (when this flag is not set) is to run McIDAS in
#       the 8-bit PseudoColor mode.
#
#       Do not set this flag unless your display does not support the
#       8-bit PseudoColor visual mode. Setting this flag may also result
#       in a significant slowdown in commands that write image or
#       graphic output.
#-------

# -displayVisualMode

#-------
# Set the geometry of the mctext window
#
# -tgeometry geometry
#
#	This option sets the geometry for the mctext window.
#	The geometry specification follows the X conventions,
#	as described in the X(1) man page.
#	The WIDTH and HEIGHT are measured in characters.
#-------

# -tgeometry $tgeometry_def

#-------
# Set the geometry of the mcimage window
#
# -igeometry geometry
#
#	This option sets the geometry for the mcimage window.
#	The geometry specification follows the X conventions,
#	as described in the X(1) man page.
#	The WIDTH and HEIGHT are ignored.  mcimage makes its
#	window the size of frame 1.
#-------

# -igeometry $igeometry_def

#-------
# Set any mcimage options.  Here we explain some typical options.
#
# -graphicsColors N
#
#	Sets the number of graphics colors to N.
#	N may be modified if there aren't enough colors available.
#
# -imageColors N
#
#	Sets the number of image colors to N.
#	N may be modified if there aren't enough colors available.
#
# -optimizeForSpeed
#
#	Display images as quickly as possible. (DEFAULT)
#	This option uses lots of server memory.
#
# -optimizeForSpace
#
#	Display images using a minimum of memory.
#	This option greatly slows down looping but more than doubles
#	the number of images which can be loaded.
#	This option is only available if the display is local, because
#	otherwise looping is slow.
#
# -manualResize
#
#	window matches frame size only under user's command (DEFAULT)
#
# -autoResize
#
#	window automatically resizes to frame size
#
#-------

# -graphicsColors 16
# -imageColors 64
# -optimizeForSpace
# -autoResize

#-------
# Specify a working directory to be used by McIDAS. 
#
# -cd DIR
#
# 	Sets the working directory to DIR.  If not specified the 
#       directory in which McIDAS was started will be used.
#
#-------

# -cd dir

#-------
# Set the behavior of the McIDAS configuration window.
#
# -config
#
#       This flag determines whether the McIDAS configuration
#       window will pop up each time McIDAS is started.  If this
#       flag is commented out the user can manually start the
#       configuration window by typing 'mcidas -config'.
#
#-------

-config

#-------
# Set the X-Cursor option.
#
# -noXCursor
#
#       This flag determines whether McIDAS will try to emulate
#       an X-window cursor.  On some X-Window servers the 
#       X-Cursor emulation can cause problems.
#
#-------

# -noXCursor



!EOF!
fi

#------
# Look for the '-config' flag in the command line arguments, if
# found then start the config GUI and exit from this script.
#------

for flag in $@
do
   if [ $flag = "-config" ]; then
   	echo "Running McIDAS configuration GUI..."
	if	[ $# -gt 1 ]
	then	`config.gui -showCMDLineArgMsg` &
	else	`config.gui` &
	fi
	exit 0
    fi
done

#-------
# If there is a .mcidasrc file, add its contents to the command line
# before any user-supplied arguments.
#-------

# count of command line arguments (not including those added from .mcidasrc)
numCMDLineArgs=$#

if	[ -f "$MCIDASRC" -a -r "$MCIDASRC" ]
then	rcopts=`sed -e 's/
$//' -e '/^[ 	]*#/d' $MCIDASRC`
	eval set X $rcopts '${1+"$@"}'; shift
fi

#-------
# Loop through all the arguments.
#
# We save arguments for mcenv and mctext in dynamically created
# variables since the arguments can accumulate and can have embedded
# white space (particularly the -c arguments for mctext).  Also, it
# is important to preserve the sequence of the -c and -cf arguments
# for mctext.
#
# We save arguments for mcimage in a single string, since the mcimage
# command is given to mctext as a single string.  Since there is no
# way to pass it arguments with embedded white space, we don't bother
# trying.
#-------

# count of mcenv args saved in mcenv_arg1..mcenv_arg$mcenv_count
mcenv_count=0
# count of mctext args saved in mctext_arg1..mctext_arg$mctext_count
mctext_count=0
# arguments to pass to mcimage
mcimage_arg=

while	[ $# != 0 ]
do
	case $1 in

	#-------
	# start configuration GUI
	#-------

	-config)
		echo "Running McIDAS configuration GUI..."
		if	[ $numCMDLineArgs -gt 0 ]
		then	`config.gui -showCMDLineArgMsg` &
		else	`config.gui` &
		fi
		exit 0
		;;

	#-------
	# change working directory
	#-------

	-cd)
		cd $2
		shift; shift
		;;

	#-------
	# whether to wait for mcenv
	#-------

	-wait)	case $# in
		1)	error_stat=TRUE
			echo "ERROR: y or n missing for $1 option"
			shift
			;;
		*)	case $2 in
			y|n)	wait_arg="$2"
				;;
			*)	error_stat=TRUE
				echo "ERROR: y or n not given to $1 option"
				;;
			esac
			shift; shift
			;;
		esac
		;;

	#-------
	# whether to use a tty-based session instead of an X-based
	# session
	#-------

	-tty)
		tty_arg="$1"
		shift
		;;

	#-------
	# -f framespec		# frame specs
	# -e extraspace		# extra frame space
	# (passed to mcenv)
	#-------

	-f	|\
	-e	)

		case $# in
		1)	error_stat=TRUE
			echo "ERROR: Argument missing for $1 option"
			shift
			;;
		*)	mcenv_count=`expr $mcenv_count + 1`
			eval "mcenv_arg$mcenv_count='$1'"
			shift
			mcenv_count=`expr $mcenv_count + 1`
			eval "mcenv_arg$mcenv_count='$1'"
			shift
			;;
		esac
		;;

	#-------
	# -c command		# command to run
	# -cf file		# file of commands to run
	#			# (undocumented)
	# -ow number		# number of text output windows
	#			# (undocumented)
	# -ih number		# number of input history lines to keep
	# -oh number		# number of output history lines to keep
	#			# in each logical output window
	# -fn font		# font name
	# -font font		# font name
	# -fontListHead fontlist# comma separated list of font names
	# -fontListTail fontlist# comma separated list of font names
	# -tgeometry geometry	# mctext geometry
	#
	# -bg color			# mctext color
	# -background color		# mctext color
	# -fg color			# mctext color
	# -foreground color		# mctext color
	#
	# -tcolbg0	color	# mctext color for background black
	# -tcolbg16	color	# mctext color for background blue
	# -tcolbg32	color	# mctext color for background green
	# -tcolbg48	color	# mctext color for background cyan
	# -tcolbg64	color	# mctext color for background red
	# -tcolbg80	color	# mctext color for background magenta
	# -tcolbg96	color	# mctext color for background yellow
	# -tcolbg112	color	# mctext color for background white
	# 
	# -tcolfg0	color	# mctext color for foreground black
	# -tcolfg1	color 	# mctext color for foreground blue
	# -tcolfg2	color 	# mctext color for foreground green
	# -tcolfg3	color 	# mctext color for foreground cyan
	# -tcolfg4	color 	# mctext color for foreground red
	# -tcolfg5	color 	# mctext color for foreground magenta
	# -tcolfg6	color 	# mctext color for foreground yellow
	# -tcolfg7	color 	# mctext color for foreground white
	# -tcolfg8	color 	# mctext color for foreground bright black
	# -tcolfg9	color 	# mctext color for foreground bright blue
	# -tcolfg10	color 	# mctext color for foreground bright green
	# -tcolfg11	color 	# mctext color for foreground bright cyan
	# -tcolfg12	color 	# mctext color for foreground bright red
	# -tcolfg13	color 	# mctext color for foreground bright magenta
	# -tcolfg14	color 	# mctext color for foreground bright yellow
	# -tcolfg15	color 	# mctext color for foreground bright white
	#
	# (passed to mctext)
	#-------

	-c			|\
	-cf			|\
	-ow			|\
	-ih			|\
	-oh			|\
	-fn			|\
	-font			|\
	-fontListHead		|\
	-fontListTail		|\
	-bg			|\
	-background		|\
	-fg			|\
	-foreground		|\
	-tcolbg0		|\
	-tcolbg16		|\
	-tcolbg32		|\
	-tcolbg48		|\
	-tcolbg64		|\
	-tcolbg80		|\
	-tcolbg96		|\
	-tcolbg112		|\
	-tcolfg0		|\
	-tcolfg1		|\
	-tcolfg2		|\
	-tcolfg3		|\
	-tcolfg4		|\
	-tcolfg5		|\
	-tcolfg6		|\
	-tcolfg7		|\
	-tcolfg8		|\
	-tcolfg9		|\
	-tcolfg10		|\
	-tcolfg11		|\
	-tcolfg12		|\
	-tcolfg13		|\
	-tcolfg14		|\
	-tcolfg15		|\
	-tg			|\
	-tge			|\
	-tgeo			|\
	-tgeom			|\
	-tgeome			|\
	-tgeomet		|\
	-tgeometr		|\
	-tgeometry		)
		case $# in
		1)	error_stat=TRUE
			echo "ERROR: Argument missing for $1 option"
			shift
			;;
		*)	mctext_count=`expr $mctext_count + 1`
			eval "mctext_arg$mctext_count='$1'"
			shift
			mctext_count=`expr $mctext_count + 1`
			eval "mctext_arg$mctext_count='$1'"
			shift
			;;
		esac
		;;

	#-------
	# -carp			# if given, replace garbage
	#			# characters in command output with
	#			# '@' characters and issue a warning
	#			# (undocumented)
	# (passed to mctext)
	#-------
	-carp)
		mctext_count=`expr $mctext_count + 1`
		eval "mctext_arg$mctext_count='$1'"
		shift
		;;

	#-------
	# display name
	# (set in environment, so that not only mcimage and mctext
	# see it, but also ASK1 and MCMENU)
	#-------

	-d	|\
	-di	|\
	-dis	|\
	-disp	|\
	-displ	|\
	-displa	|\
	-display)
		export DISPLAY; DISPLAY="$2"
		echo "DISPLAY set to '$DISPLAY'"
		shift; shift
		;;

	#-------
	# help
	#-------

	-help|-h|-?)
sed '/^#/d' <<EOHELP
-f COUNT@LINESxELEMS	Allocate COUNT frames of the given size.
			This option may be repeated.
			(default: given in \$HOME/.mcidasrc)

-e COUNT		Allocate pool of COUNT bytes for MAKFRM to use.
			If COUNT ends in 'k' it counts KB (e.g. '256k').
			If COUNT ends in 'm' it counts MB (e.g. '10m').
			(default: no extra frame space)

-c 'command'		Run the given command on startup.
			This option may be repeated.
			(default: given in \$HOME/.mcidasrc)

-font fontname		Name of font to be used in the text window.
			(default: $fontname_def)

-ih number		Number of commands to remember.
			(default: $cmdhistlen_def)

-oh number		Number of text output lines to remember.
			(default: $winhistlen_def)

-display name		Name of X server to use.
			(default: $DISPLAY_def)

-tgeometry geometry	Geometry of text window.
			(default: $tgeometry_def)

-igeometry geometry	Geometry of image window.
			(default: $igeometry_def)

mcimage option		passed to mcimage
			(default: no mcimage options)
EOHELP
		exit 0
		;;

	#-------
	# -verbose - recognized here, also passed to mcimage
	#-------

	-verbose)
		verbose_arg="$1"
		mcimage_arg="$mcimage_arg $1"
		shift
		;;

	#-------
	# unrecognized option - passed to mcimage
	# if the first arg after the option is not empty and
	# does not begin with '-', it is also passed to mcimage
	#-------

	-*)
		mcimage_arg="$mcimage_arg $1"
		shift
		case $1 in
		-*)	;;
		?*)	mcimage_arg="$mcimage_arg $1"
			shift
			;;
		esac
		;;

	#-------
	# error
	#-------

	*)	error_stat=TRUE
		echo "ERROR: unexpected argument '$1'"
		shift
		;;
	esac
done


#-------
# finish up
#-------

# check DISPLAY
#
# Formerly, this was checked before parsing the options.  Now we
# have to check after parsing the options, because DISPLAY could
# have been set with -display, and because the -tty option could
# have been given.
#
# If -tty was given, make sure DISPLAY is not set.  This makes
# mctext go into full-screen (curses) mode.
#
# If -tty was NOT given, make sure DISPLAY is set.  This prevents
# people from accidentally getting a full-screen (curses) mcidas.

case $tty_arg in
?*)	unset DISPLAY
	;;
*)	case $DISPLAY in
	'')     error_stat=TRUE
		echo "ERROR: 'DISPLAY' variable must be set to execute mcidas"
		;;
	esac
	;;
esac


# By default, we don't cd anywhere.
# However, for backwards compatibility, if there is no MCPATH, we
# cd to the first writable directory in MCPATH before starting
# mcidas.
do_cd=

case $MCPATH in
'')
	#-------
	# Make sure the user has the correct directory structure for
	# using McIDAS-X.
	#-------

	for dir in	$HOME/mcidas		\
			$HOME/mcidas/data	\
			$HOME/mcidas/help
	do	if	[ ! -d "$dir" ]
		then	echo "Making directory $dir"
			if	mkdir "$dir"
			then	:
			else	echo "ERROR: Could not make directory $dir"
				exit 1
			fi
		fi
	done

	#-------
	# Initial default MCPATH setting.
	#-------

	export MCPATH;	MCPATH="$HOME/mcidas/data:$HOME/mcidas/help"

	#-------
	# Set mcidashome to the home directory of the mcidas account.
	#-------

	# We can't use ~mcidas since /bin/sh doesn't do tilde expansion.
	# Not all systems have 'logdir' either, so we do it ourselves
	# with sed. Interix on the other hand doesn't contain the file 
	# /etc/passwd, so don't attempt to use it, but rather default to
	# tilde expansion using the csh.

	if [ -r /etc/passwd ]; then

	 mcidashome=`sed -n "/^mcidas:/ s,.*:\([^:]*\):[^:]*$,\1,p" /etc/passwd`

	fi

	# If that didn't work, then try to use csh for tilde expansion
	# to determine the home directory of the mcidas account.
	# Unlike grepping /etc/passwd, this should work even on systems
	# that use yellow pages.

	case $mcidashome in
	/*)	;;
	*)	mcidashome=`csh -f -c "echo ~mcidas" 2>/dev/null`
		;;
	esac

	# If we can't find a mcidas account, issue a warning

	case $mcidashome in
	/*)	;;
	*)	echo "WARNING: Cannot locate a mcidas account."
		;;
	esac

	#-------
	# If we can locate the mcidas account, add the appropriate
	# directories to MCPATH.
	#-------

	case $mcidashome in
	/*)	MCPATH="$MCPATH:$mcidashome/data:$mcidashome/help"
		;;
	esac

	echo "WARNING: No MCPATH environment variable."
	echo "Using MCPATH=$MCPATH"
	do_cd=true
	;;
*:*)	# Here we assume we have a legitimate MCPATH,
	# since it has a : in it.
	;;
*)	# With no colons, it's probably an old-style MCPATH setting
	# for HELP
	if	[ -d "$MCPATH" ]
	then	echo "ERROR: Using old-style MCPATH for HELP."
		exit 1
	fi
	;;
esac

# mcpath - space-separated version of MCPATH

mcpath=`IFS=":$IFS"; set X $MCPATH; shift; echo $*`

# Check that all MCPATH directories are absolute.
# The underlying code does not require this check, but it
# helps to prevent confusion.
# Note that this loop does not catch cases where MCPATH
# contains an implied '.' due to leading or trailing
# colons, or embedded pairs of colons.

for d in $mcpath
do
	case $d in
	/*)	;;
	'~'*)	error_stat=TRUE
		echo "ERROR: MCPATH directory '$d' refers to a nonexistent account."
		;;
	*)	error_stat=TRUE
		echo "ERROR: MCPATH directory '$d' may not be relative.  It should have a name beginning with '/'"
		;;
	esac
done

# Check for implied '.' directories in MCPATH.
# The underlying code does not require this check, but it
# helps to prevent confusion.

case $MCPATH in
:*)	error_stat=TRUE
	echo "ERROR: MCPATH begins with an implied '.' directory"
	;;
esac
case $MCPATH in
*::*)	error_stat=TRUE
	echo "ERROR: MCPATH contains an implied '.' directory"
	;;
esac
case $MCPATH in
*:)	error_stat=TRUE
	echo "ERROR: MCPATH ends with an implied '.' directory"
	;;
esac

# Locate the first writeable directory in MCPATH.
# That's where new files will go.

dir=
for d in $mcpath
do
	if	[ -d "$d" -a -w "$d" -a -x "$d" ]
	then	dir=$d
		break
	fi
done
case $dir in
'')	error_stat=TRUE
	echo "ERROR: MCPATH contains no writeable directory"
	;;
esac

case $do_cd in
?*)
	if	cd $dir
	then	echo "Current directory is $dir"
	else	echo "ERROR: cannot cd to $dir"
		exit 1
	fi
	;;
esac

if	[ ! -r . ]
then	error_stat=TRUE
	echo "ERROR: current directory is not readable"
fi

if	[ ! -w . ]
then	error_stat=TRUE
	echo "ERROR: current directory is not writable"
fi

if	[ ! -x . ]
then	error_stat=TRUE
	echo "ERROR: current directory is not searchable"
fi

case $error_stat in
TRUE)
	#-------
	# All the spots above that set error_stat print a message,
	# so this message is superfluous.
	#-------
	exit 1
	;;
esac

#-------
# set LDR_CTRL on AIX
#-------

UNAMES=`uname -s`
if [ "$UNAMES" = "AIX" -a -z "$LDR_CNTRL" ]; then
	SYSMEM=`/usr/sbin/lsattr -El sys0 2>/dev/null |grep realmem |awk '{print $2}'`
	if [ -n "$SYSMEM" ]; then
		if [ $SYSMEM -ge 1048576 ]; then
			# 1024mb max heap+stack
			LDR_CNTRL=0x40000000
		elif [ $SYSMEM -ge 524288 ]; then
			# 512mb max heap+stack
			LDR_CNTRL=0x20000000
		else
			# Default heap+stack
			LDR_CNTRL=0x00000000
		fi
                export LDR_CNTRL=MAXDATA=$LDR_CNTRL
	fi
fi

#-------
# set LD_LIBRARY_PATH if it looks like it might be needed
#     (but NOT on solaris, where we want /opt/csw/lib)
#-------

if [ -f /usr/local/lib/libgcc_s.so.1 -a "$UNAMES" != "SunOS" ]; then
	if [ -z "${LD_LIBRARY_PATH}" ]; then
		LD_LIBRARY_PATH=/usr/local/lib
	else
		LD_LIBRARY_PATH=/usr/local/lib:${LD_LIBRARY_PATH}
	fi
	export LD_LIBRARY_PATH
fi

#-------
# Set PATH on Ubuntu systems
#-------
if [ `grep "Ubuntu" /etc/os-release 2>/dev/null |wc -l` -ne 0 ]; then
	PATH=/home/$USER/mcidas/bin:/home/mcidas/bin:/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin	
	export PATH
fi

#-------
# clean up session files for any defunct sessions
#-------

mcclean &

#-------
# Construct the command by setting our positional parameters.
# We do this to preserve embedded white space in the arguments.
#-------

# Clear the positional parameters.

set X; shift

# Add the mcenv arguments.

set X ${1+"$@"} mcenv
shift

# if DISPLAY is not set, we do not run mcimagem or mcimage
# If it is, start mcimagem and tell mcenv how to kill it
# if something goes wrong.

case $DISPLAY in
?*)
	# start the progress window
	mcimagem_out=`mcimagem -top`
	eval `	set X $mcimagem_out
		shift
		echo "mcimagem_wid='\$1'"
		echo "mcimagem_pid='\$2'"`

	# echo "mcimagem_wid='$mcimagem_wid'"
	# echo "mcimagem_pid='$mcimagem_pid'"

	# Add a -k argument to tell mcenv how to kill mcimagem
	# if something goes wrong.
	# This needs to be the FIRST argument to mcenv.  If it
	# were not, it would be possible for the other arguments
	# to be erroneous in such a way as to prevent mcenv from
	# being able to kill mcimagem.

	set X ${1+"$@"} -k "$mcimagem_pid"
	shift
	;;
esac

# Add any other mcenv arguments.

i=0
while	[ $i -lt $mcenv_count ]
do
	i=`expr $i + 1`
	eval set X '${1+"$@"}' "\"\$mcenv_arg$i\""
	shift
done

# Add the mctext arguments.

set X ${1+"$@"} mctext
shift

# if DISPLAY is not set, we do not run mcimagem or mcimage

case $DISPLAY in
?*)
	# Add a -c argument to tell mctext to run mcimage.
	# The ! means to run it with the shell,
	# the @ means not to echo it,
	# and the exec saves a process.
	# The -iw tells mctext to wait for mcimage before
	# mapping its own window.

	set X ${1+"$@"} -iw -c "!@exec mcimage$mcimage_arg -progressID $mcimagem_wid"
	shift
	;;
esac

i=0
while	[ $i -lt $mctext_count ]
do
	i=`expr $i + 1`
	eval set X '${1+"$@"}' "\"\$mctext_arg$i\""
	shift
done

case $verbose_arg in
?*)
	for x
	do
		echo "'$x'" >&2
	done
	;;
esac

wait	# for mcclean

# Start the session.

# If DISPLAY is not set, we ALWAYS want to wait for mcenv.

case $DISPLAY in
'')	wait_arg=y
	;;
esac

# If the wait argument is not set,
# and we are under ARSAP accounting,
# we wait for mcenv.
# Otherwise, we don't.

case $wait_arg in
'')
	case $ARSAP_CURRPROJ in
	?*)	wait_arg=y
		;;
	*)	wait_arg=n
		;;
	esac
	;;
esac

case $wait_arg in
y)	exec "$@"
	;;
*)	"$@" &
	;;
esac
